#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import os.path
import httplib
import urllib
import json
import socket
import time
from pprint import pprint  # to debug...
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt
from PyQt4.QtGui import QMessageBox, QIcon

PASS_FILE = 'password.sav'

class RefreshConnect(QtCore.QThread):
    clientDeconnected = QtCore.pyqtSignal()
    clientConnected = QtCore.pyqtSignal()
    
    def refreshConnect(self, login, passwordDigest):
        print 'refreshConnect'
        paramsString = urllib.urlencode({
            'action': 'refresh',
            'login': login,
            'password_digest': passwordDigest,
            'time': int(time.time())
        })
        try:
            headers = {"Content-type": "application/x-www-form-urlencoded"}
            conn = httplib.HTTPSConnection('controller.mobile.lan')
            conn.request('POST', '/portal_api.php', paramsString, headers)
            res = conn.getresponse()
            data = json.loads(res.read())
            if 'error' in data:
                if data['error']['code'] == 'error_refresh_disconnect':
                    self.clientDeconnected.emit()
                else:
                    print 'Error : %s' % data['error']['code']
            else:
                self.clientConnected.emit()
        except socket.gaierror:
            print 'PAS DE CONNNECTION'
    
    
    def run(self):
        while True:
            login = self.parent().login.text()
            passwordDigest = self.parent().passwordDigest
            if login != '' and self.parent().autoConnect:
                if passwordDigest == '':
                    self.clientDeconnected.emit()
                else:
                    self.refreshConnect(login, passwordDigest)
                    print '-'
            self.sleep(10)



class Main(QtGui.QMainWindow):
    passwordDigest = ''
    autoConnect = True
    
    def __init__(self):
        QtGui.QMainWindow.__init__(self)
        self.setWindowTitle('Patton Connect')
        self.setFixedSize(250, 120)
        self.drawWidgets()
        
        self.refreshConnect = RefreshConnect(self)
        self.refreshConnect.start()
        self.refreshConnect.clientDeconnected.connect(self.clientDeconnected)
        self.refreshConnect.clientConnected.connect(self.clientConnected)
        
        self.timer = QtCore.QTimer(self)
        self.timer.setInterval(2000)
        self.timer.timeout.connect(self.reloadThread)
        self.timer.start()
        
        if os.path.isfile(PASS_FILE):
            with open(PASS_FILE) as f:
                login, password = f.readline().strip().split(':')
                self.login.setText(login)
                self.password.setText(password)
    
    
    def reloadThread(self):
        if not self.refreshConnect.isRunning():
            self.refreshConnect.start()
    
    
    def clientConnected(self):
        self.loginWidget.hide()
        self.logoutWidget.show()
    
    
    def clientDeconnected(self):
        self.connection()
        self.loginWidget.show()
        self.logoutWidget.hide()
    
    
    def connection(self):
        self.autoConnect = True
        login, password = self.login.text(), self.password.text()
        with open(PASS_FILE, 'w+') as f:
            f.write('%s:%s' % (login, password))
        
        paramsString = urllib.urlencode({
            'action': 'authenticate',
            'login': login,
            'password': password,
            'policy_accept': 'true'
        })
        headers = {"Content-type": "application/x-www-form-urlencoded"}
        try:
            conn = httplib.HTTPSConnection('controller.mobile.lan')
            conn.request('POST', '/portal_api.php', paramsString, headers)
            res = conn.getresponse()
            data = json.loads(res.read())
            self.passwordDigest = data['user']['passwordDigest']['value']
            if self.passwordDigest != '':
                self.loginWidget.hide()
                self.logoutWidget.show()
        except socket.gaierror:
            print 'PAS DE CONNNECTION'
    
    
    def deconnection(self):
        login = self.login.text()
        passwordDigest = self.passwordDigest
        paramsString = urllib.urlencode({
            'action': 'disconnect',
            'login': login,
            'password_digest': passwordDigest
        })
        headers = {"Content-type": "application/x-www-form-urlencoded"}
        conn = httplib.HTTPSConnection('controller.mobile.lan')
        conn.request('POST', '/portal_api.php', paramsString, headers)
        res = conn.getresponse()
        data = json.loads(res.read())
        pprint(data)
        self.passwordDigest = ''
        self.autoConnect = False
        self.loginWidget.show()
        self.logoutWidget.hide()
    
    
    def drawWidgets(self):
        self.login = QtGui.QLineEdit()
        self.password = QtGui.QLineEdit()
        self.password.setEchoMode(QtGui.QLineEdit.Password)
        self.btn = QtGui.QPushButton('Connexion')
        self.btn.clicked.connect(self.connection)
        
        form = QtGui.QFormLayout()
        form.addRow('Login', self.login)
        form.addRow('Mot de passe', self.password)
        form.addRow(self.btn)
        
        self.loginWidget = QtGui.QWidget()
        self.loginWidget.setLayout(form)
        
        self.logoutBtn = QtGui.QPushButton(u'Déconnexion')
        self.logoutBtn.clicked.connect(self.deconnection)
        
        form2 = QtGui.QFormLayout()
        form2.addRow(self.logoutBtn)
        
        self.logoutWidget = QtGui.QWidget()
        self.logoutWidget.setLayout(form2)
        self.logoutWidget.hide()
        
        mainLayout = QtGui.QVBoxLayout()
        mainLayout.setAlignment(Qt.AlignCenter)
        mainLayout.addWidget(self.loginWidget)
        mainLayout.addWidget(self.logoutWidget)
        
        mainWidget = QtGui.QWidget()
        mainWidget.setLayout(mainLayout)
        
        self.setCentralWidget(mainWidget)


app = QtGui.QApplication(sys.argv)
window = Main()
window.show()
sys.exit(app.exec_())